/*******************************************************}
{                                                       }
{       RichView                                        }
{       Demo: how to create MIME document.              }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************/


/*
  This demo shows how to create MIME (Multipurpose Internet Mail Extensions)
  document.
  These documents contain both HTML and images in one file.
  File extension:
  *.eml - e-mail file, can be opened, for example, by Outlook Express.
  *.mht - web archive, can be opened, for example, by Internet Explorer.
  These files are essentially the same.

  This demo uses The Delphi Inspiration's DIMime unit for base64 encoding
  (freeware).
  Note: usually, base64 is used only for images and attachments. The main
  document is encoded as quoted-printable.
  For the simplicity reason, this demo encodes ALL parts as base64
  (thus making HTML part unreadable by humans).

  DIMime (c) Copyright Ralf Junker
  http://www.yunqa.de/delphi/doku.php/products/mime/
  Installation is included in this demo.

  Additional information about MIME:
  RFC 2045
  http://www.ietf.org/rfc/rfc2045.txt
  RFC 2046
  http://www.ietf.org/rfc/rfc2046.txt
  RFC 2047
  http://www.ietf.org/rfc/rfc2047.txt
  RFC 2048
  http://www.ietf.org/rfc/rfc2048.txt
  RFC 2049
  http://www.ietf.org/rfc/rfc2049.txt
*/


//---------------------------------------------------------------------------

#include <vcl.h>
#include <stdio.h>
#pragma hdrstop

#include "Unit1.h"
#include "DIMime.hpp"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "RichView"
#pragma link "RVEdit"
#pragma link "RVScroll"
#pragma link "RVStyle"
#pragma resource "*.dfm"
TForm1 *Form1;

/* Actually, this demo converts all pictures to Jpegs, so only TJpegImage can
  occur here. But you can use RV_RegisterHTMLGraphicFormat to allow other
  graphic formats in HTML */
AnsiString GetImageContentType(TGraphic* gr)
{
  if (gr->InheritsFrom(__classid(TJPEGImage)))
    return "image/jpeg";
  else if (gr->InheritsFrom(__classid(Graphics::TBitmap)))
    return "image/bmp";
  else if (gr->InheritsFrom(__classid(TIcon)))
    return "image/x-icon";
  else if (gr->InheritsFrom(__classid(TMetafile)))
    return "application/octet-stream";
  /*
  else if (gr->InheritsFrom(__classid(TPNGObject)))
    return "image/png";
  else if (gr->InheritsFrom(__classid(TGifImage)))
    return "image/gif";
  */
  else
    throw Exception("Unknown graphic format");
}
//---------------------------------------------------------------------------
// THTMLImageItem
//---------------------------------------------------------------------------
__fastcall THTMLImageItem::THTMLImageItem(TCollection* Collection)
: TCollectionItem(Collection)
{
  Stream = new TMemoryStream;
}
//---------------------------------------------------------------------------
__fastcall THTMLImageItem::~THTMLImageItem()
{
  delete Stream;
}
//---------------------------------------------------------------------------
// THTMLImagesCollection
//---------------------------------------------------------------------------
__fastcall THTMLImagesCollection::THTMLImagesCollection()
: TCollection(__classid(THTMLImageItem))
{
}
//---------------------------------------------------------------------------
THTMLImageItem* __fastcall THTMLImagesCollection::GetItem(int Index)
{
  return (THTMLImageItem*)(TCollection::GetItem(Index));
}
//---------------------------------------------------------------------------
void __fastcall THTMLImagesCollection::SetItem(int Index, const THTMLImageItem* Value)
{
  TCollection::SetItem(Index, (TCollectionItem*)Value);
}
//---------------------------------------------------------------------------
// TForm1
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{

}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
  HTMLImages = new THTMLImagesCollection;

  RichViewEdit1->Clear();
  RichViewEdit1->AddBreakEx(4, rvbsLine, clBlue);
  RichViewEdit1->AddNL("MIME File Test", 1, 1);
  Graphics::TBitmap *bmp = new Graphics::TBitmap;
  bmp->Assign(Image1->Picture->Graphic);
  RichViewEdit1->AddPictureEx("", bmp, 1, rvvaBaseline);
  RichViewEdit1->AddNL("This example shows how to save HTML and images in one file.", 0, 0);
  RichViewEdit1->AddBreakEx(4, rvbsLine, clBlue);
  RichViewEdit1->Format();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormDestroy(TObject *Sender)
{
  delete HTMLImages;        
}
//---------------------------------------------------------------------------
/* RichViewEdit1->OnSaveImage2: storing image in the HTMLImages collection instead
  of files. */
void __fastcall TForm1::RichViewEdit1SaveImage2(TCustomRichView *Sender,
      TGraphic *Graphic, TRVSaveFormat SaveFormat, const AnsiString Path,
      const AnsiString ImagePrefix, int &ImageSaveNo, AnsiString &Location,
      bool &DoDefault)
{
  if (SaveFormat!=rvsfHTML)
    return;
  AnsiString Ext = GraphicExtension((TGraphicClass)(Graphic->ClassType()));
  Location = Format("image%d.%s", ARRAYOFCONST((ImageSaveNo, Ext)));
  ImageSaveNo++;
  THTMLImageItem* Item = (THTMLImageItem*)(HTMLImages->Add());
  Graphic->SaveToStream(Item->Stream);
  Item->Name = Location;
  Item->ContentType = GetImageContentType(Graphic);
  Location = "cid:"+Location;
  DoDefault = false;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Button1Click(TObject *Sender)
{

  if (! SaveDialog1->Execute())
    return;

  HTMLImages->Clear();

  // saving HTML
  TStringStream* Stream = new TStringStream("");
  RichViewEdit1->SaveHTMLToStreamEx(Stream, "", "Web Archive Demo", "", "", "", "",
    TRVSaveOptions() << rvsoUseCheckpointsNames << rvsoUTF8);
  AnsiString s = MimeEncodeString(Stream->DataString);
  delete Stream;
  // now s contains HTML file without images, base64-encoded. saving it in MIME
  AnsiString boundary = "----=_BOUNDARY_LINE_";
  FILE* F = fopen(SaveDialog1->FileName.c_str(), "wt");
  fprintf(F, "From: <Saved by TRICHVIEW.COM Demo>\n"
    "Subject: MIME Test\n"
    "MIME-Version: 1.0\n"
    "Content-Type: multipart/related;\n"
    "\tboundary=\"");
  fprintf(F, boundary.c_str());
  fprintf(F, "\"\nX-Unsent: 1\n\n"
    "This is a multi-part message in MIME format.\n\n"
    "--");
  fprintf(F, boundary.c_str());
  fprintf(F, "\nContent-Type: text/html;\n"
    "\tcharset=\"utf-8\"\n"
    "Content-Transfer-Encoding: base64\n\n");
  fprintf(F, s.c_str());
  fprintf(F, "\n");
  // saving images
  for (int i=0; i<HTMLImages->Count; i++)
  {
    fprintf(F, "\n--");
    fprintf(F, boundary.c_str());
    fprintf(F, "\n");
    s.SetLength(HTMLImages->Items[i]->Stream->Size);
    HTMLImages->Items[i]->Stream->Position = 0;
    HTMLImages->Items[i]->Stream->ReadBuffer(s.c_str(), s.Length());
    s = MimeEncodeString(s);
    fprintf(F, "Content-Type: ");
    fprintf(F, HTMLImages->Items[i]->ContentType.c_str());
    fprintf(F, "\n\tName=\"");
    fprintf(F, HTMLImages->Items[i]->Name.c_str());
    fprintf(F,  "\"\nContent-Transfer-Encoding: base64\n"
      "Content-ID: <");
    fprintf(F, HTMLImages->Items[i]->Name.c_str());
    fprintf(F, ">\n\n");
    fprintf(F, s.c_str());
    fprintf(F, "\n");
  }
  fprintf(F, "--");
  fprintf(F, boundary.c_str());
  fprintf(F, "--\n");
  fclose(F);
  HTMLImages->Clear();
}
//---------------------------------------------------------------------------