
{*******************************************************}
{                                                       }
{       RichView                                        }
{       Demo: how to create MIME document.              }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************}


{
  Version 2.1


  This demo shows how to send HTML e-mail.
  This e-mail includes text representation, HTML, images and attached files.

  This demo uses The Delphi Inspiration's DIMime unit for base64 encoding
  (freeware).
  Note: usually, base64 is used only for images and attachments. The main
  document is encoded as quoted-printable.
  For the simplicity reason, this demo encodes ALL parts as base64
  (thus making text and HTML part unreadable by humans).

  DIMime (c) Copyright Ralf Junker
  http://www.yunqa.de/delphi/doku.php/products/mime/
  Installation is included in this demo.

  Additional information about MIME:
  RFC 2045
  http://www.ietf.org/rfc/rfc2045.txt
  RFC 2046
  http://www.ietf.org/rfc/rfc2046.txt
  RFC 2047
  http://www.ietf.org/rfc/rfc2047.txt
  RFC 2048
  http://www.ietf.org/rfc/rfc2048.txt
  RFC 2049
  http://www.ietf.org/rfc/rfc2049.txt
}

unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, RVStyle, RVScroll, RichView, RVEdit, DIMime, Jpeg,
  ExtCtrls, CRVData, RVUni, IdBaseComponent, IdComponent,
  IdTCPConnection, IdTCPClient, IdMessageClient, IdSMTP, IdMessage,
  RVTypes;

type

  { This class will be used to store information about one picture while
    saving to HTML }
  THTMLImageItem = class (TCollectionItem)
    public
      Stream: TMemoryStream;
      Name, ContentType: TRVAnsiString;
      constructor Create(Collection: TCollection); override;
      destructor Destroy; override;
  end;
  { Collection of THTMLImageItem }
  THTMLImagesCollection = class (TCollection)
  private
    function GetItem(Index: Integer): THTMLImageItem;
    procedure SetItem(Index: Integer; const Value: THTMLImageItem);
    public
      constructor Create;
      property Items[Index: Integer]: THTMLImageItem read GetItem write SetItem; default;
  end;

  TfrmMain = class(TForm)
    RVStyle1: TRVStyle;
    btnSend: TButton;
    Image1: TImage;
    IdSMTP1: TIdSMTP;
    IdMessage1: TIdMessage;
    gbFrom: TGroupBox;
    lblFromName: TLabel;
    txtFromName: TEdit;
    lblFromEMail: TLabel;
    txtFromEMail: TEdit;
    gbTo: TGroupBox;
    lblToName: TLabel;
    lblToEMail: TLabel;
    txtToName: TEdit;
    txtToEMail: TEdit;
    gbMessage: TGroupBox;
    Label1: TLabel;
    txtSubject: TEdit;
    RichViewEdit1: TRichViewEdit;
    Label2: TLabel;
    gbServer: TGroupBox;
    lblHost: TLabel;
    lblPort: TLabel;
    txtHost: TEdit;
    txtPort: TEdit;
    lblUserID: TLabel;
    lblPassword: TLabel;
    txtPassword: TEdit;
    txtUserID: TEdit;
    gbFiles: TGroupBox;
    txtFiles: TMemo;
    Label3: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure RichViewEdit1SaveImage2(Sender: TCustomRichView;
      Graphic: TGraphic; SaveFormat: TRVSaveFormat; const Path,
      ImagePrefix: String; var ImageSaveNo: Integer; var Location: String;
      var DoDefault: Boolean);
    procedure btnSendClick(Sender: TObject);
  private
    { Private declarations }
    HTMLImages: THTMLImagesCollection;
    HTMLImageCount: Integer;
    Attachments: TStringList;
    function GetEMail: TRVAnsiString;
    procedure AddMainContentType(SL: TStrings);
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

const
  crlf = #13#10;

implementation

{$R *.dfm}

{ Actually, this demo converts all pictures to Jpegs, so only TJpegImage can
  occur here. But you can use RV_RegisterHTMLGraphicFormat to allow other
  graphic formats in HTML }
function GetImageContentType(gr: TGraphic): TRVAnsiString;
begin
  if gr is TJpegImage then
    Result := 'image/jpeg'
  else if gr is TBitmap then
    Result := 'image/bmp'
  else if gr is TIcon then
    Result := 'image/x-icon'
  else if gr is TMetafile then
    Result := 'application/octet-stream'
  {
  else if gr is TPNGObject then
    Result := 'image/png'
  else if gr is TGifImage then
    Result := 'image/gif'
  }
  else
    raise Exception.Create('Unknown graphic format');
end;

{=============================== THTMLImageItem ===============================}
constructor THTMLImageItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  Stream := TMemoryStream.Create;
end;
{------------------------------------------------------------------------------}
destructor THTMLImageItem.Destroy;
begin
  Stream.Free;
  inherited;
end;
{============================ THTMLImagesCollection ===========================}
constructor THTMLImagesCollection.Create;
begin
  inherited Create(THTMLImageItem);
end;
{------------------------------------------------------------------------------}
function THTMLImagesCollection.GetItem(Index: Integer): THTMLImageItem;
begin
  Result := THTMLImageItem(inherited GetItem(Index));
end;
{------------------------------------------------------------------------------}
procedure THTMLImagesCollection.SetItem(Index: Integer;
  const Value: THTMLImageItem);
begin
  inherited SetItem(Index, Value);
end;
{================================= TForm1 =====================================}
procedure TfrmMain.FormCreate(Sender: TObject);
var bmp: TBitmap;
begin
  // This is a list of file names to attach to the message
  // It is empty in this demo
  Attachments := TStringList.Create;
  // Creating sample document
  RichViewEdit1.Clear;
  RichViewEdit1.AddBreakEx(4, rvbsLine, clBlue);
  RichViewEdit1.AddNL('HTML E-mail Test', 1, 1);
  bmp := TBitmap.Create;
  bmp.Assign(Image1.Picture.Graphic);
  RichViewEdit1.AddPictureEx('', bmp, 1, rvvaBaseline);
  RichViewEdit1.AddNL('This example shows how to send HTML e-mail using Indy components.', 0, 0);
  RichViewEdit1.AddBreakEx(4, rvbsLine, clBlue);
  RichViewEdit1.Format;
  RichViewEdit1.BackgroundBitmap := nil;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  Attachments.Free;
end;
{------------------------------------------------------------------------------}
{ RichViewEdit1.OnSaveImage2: storing image in the HTMLImages collection instead
  of files. }
procedure TfrmMain.RichViewEdit1SaveImage2(Sender: TCustomRichView;
  Graphic: TGraphic; SaveFormat: TRVSaveFormat; const Path,
  ImagePrefix: String; var ImageSaveNo: Integer; var Location: String;
  var DoDefault: Boolean);
var gr: TGraphic;
    bmp: TBitmap;
begin
  if SaveFormat<>rvsfHTML then
    exit;
  if not (Graphic is TJPEGImage) and not RV_IsHTMLGraphicFormat(Graphic) then begin
    bmp := TBitmap.Create;
    try
      bmp.Assign(Graphic);
    except
      bmp.Width := Graphic.Width;
      bmp.Height := Graphic.Height;
      bmp.Canvas.Draw(0,0, Graphic);
    end;
    gr := TJPEGImage.Create;
    gr.Assign(bmp);
    bmp.Free;
    end
  else
    gr := Graphic;

  Location := Format('image%d.%s', [ImageSaveNo, GraphicExtension(TGraphicClass(gr.ClassType))]);
  inc(ImageSaveNo);
  with HTMLImages.Add as THTMLImageItem do
  begin
    gr.SaveToStream(Stream);
    Name := TRVAnsiString(Location);
    ContentType := GetImageContentType(gr);
  end;
  Location := 'cid:'+Location;
  DoDefault := False;
  if gr<>Graphic then
    gr.Free;
end;
{------------------------------------------------------------------------------}
const
  BOUNDARY_TEXT_AND_HTML        = '----=_TEXT_AND_HTML_BOUNDARY_';
  BOUNDARY_MESSAGE_AND_PICTURES = '----=_MESSAGE_AND_PICS_BOUNDARY_';
  BOUNDARY_MAIN_AND_ATTACHMENTS = '----=_MAIN_AND_ATTACHMENTS_BOUNDARY_';
// 1) Returns text for assigning to Body.Text.
// 2) Assigns HTMLImageCount variable (will be used in procedure AddMainContentType)
function TfrmMain.GetEMail: TRVAnsiString;
var Stream: TMemoryStream;
    Stream2: TMemoryStream;
    Stream3: TFileStream;
    ws: WideString;
    s: TRVRawByteString;
    s2: TRVAnsiString;
    i: Integer;
begin
  HTMLImages := THTMLImagesCollection.Create;
  // saving text
  Stream2 := TMemoryStream.Create;
  RichViewEdit1.SaveTextToStreamW('', Stream2, 80, False, False);
  Stream2.Position := 0;
  SetLength(ws, Stream2.Size div 2);
  if Stream2.Size<>0 then
    Stream2.ReadBuffer(Pointer(ws)^, Stream2.Size);
  s2 := MimeEncodeString(Utf8Encode(ws));
  Stream2.Free;
  // saving HTML
  Stream := TMemoryStream.Create;
  RichViewEdit1.SaveHTMLToStreamEx(Stream, '', 'Web Archive Demo', '', '', '', '',
    [rvsoUseCheckpointsNames, rvsoUTF8]);
  Stream.Position := 0;
  SetLength(s, Stream.Size);
  Stream.ReadBuffer(PRVAnsiChar(s)^, Length(s));
  s := MimeEncodeString(s);
  Stream.Free;

  Result := 'This is a multi-part message in MIME format.'+CRLF+CRLF;

  if Attachments.Count>0 then begin
    Result := Result + '--'+BOUNDARY_MAIN_AND_ATTACHMENTS+CRLF;
    if HTMLImages.Count>0 then
      Result := Result + 'Content-Type: multipart/related;'+CRLF+
        #9+'type="multipart/alternative";'+CRLF+
        #9+'boundary="'+BOUNDARY_MESSAGE_AND_PICTURES+'"'+CRLF+CRLF+
        '--'+BOUNDARY_MESSAGE_AND_PICTURES+CRLF;
    Result := Result + 'Content-Type: multipart/alternative;'+CRLF+
      #9+'boundary="'+BOUNDARY_TEXT_AND_HTML+'"'+CRLF+CRLF+
      '--'+BOUNDARY_TEXT_AND_HTML+CRLF;
    end
  else begin
    if HTMLImages.Count>0 then begin
      Result := Result + '--'+BOUNDARY_MESSAGE_AND_PICTURES+CRLF;
      Result := Result + 'Content-Type: multipart/alternative;'+CRLF+
        #9+'boundary="'+BOUNDARY_TEXT_AND_HTML+'"'+CRLF+CRLF+
        '--'+BOUNDARY_TEXT_AND_HTML+CRLF;
      end
    else
      Result := Result + '--'+BOUNDARY_TEXT_AND_HTML+CRLF;
  end;

  // now s contains HTML without images, base64-encoded,
  // s2 contains text, base64-encoded.
  // Saving them in MIME

  Result := Result +
       'Content-Type: text/plain;'+CRLF+
       #9'charset="utf-8"'+CRLF+
       'Content-Transfer-Encoding: base64'+CRLF+CRLF+
       s2+CRLF+CRLF+
       '--'+BOUNDARY_TEXT_AND_HTML+CRLF+
       'Content-Type: text/html;'+CRLF+
       #9'charset="utf-8"'+CRLF+
       'Content-Transfer-Encoding: base64'+CRLF+CRLF+
       s+CRLF+CRLF+
       '--'+BOUNDARY_TEXT_AND_HTML+'--'+CRLF;

  // saving images
  for i := 0 to HTMLImages.Count-1 do begin
    s2 := CRLF+
          '--'+BOUNDARY_MESSAGE_AND_PICTURES+CRLF;
    SetLength(s, HTMLImages[i].Stream.Size);
    HTMLImages[i].Stream.Position := 0;
    HTMLImages[i].Stream.ReadBuffer(PRVAnsiChar(s)^, Length(s));
    s := MimeEncodeString(s);
    s2 := s2+'Content-Type: '+HTMLImages[i].ContentType+';'+CRLF+
      #9'name="'+HTMLImages[i].Name+'"'+CRLF+
      'Content-Transfer-Encoding: base64'+CRLF+
      'Content-ID: <'+HTMLImages[i].Name+'>'+CRLF+CRLF+
      s+CRLF;
    Result := Result+s2;
  end;

  if HTMLImages.Count>0 then
    Result := Result + CRLF+'--'+BOUNDARY_MESSAGE_AND_PICTURES+'--'+CRLF;

  // saving attached files
  { this demo does not encode file names, so the result may be incorrect
    if file names have non-English characters }
  for i := 0 to Attachments.Count-1 do begin
    s := TRVRawByteString(ExtractFileName(Attachments[i]));
    s2 := CRLF+ '--'+BOUNDARY_MAIN_AND_ATTACHMENTS+CRLF+
    'Content-Type: application/octet-stream;'+CRLF+#9'name="'+s+'"'+CRLF+
    'Content-Disposition: attachment;'+CRLF+#9'filename="'+s+'"'+CRLF+
    'Content-Transfer-Encoding: base64'+CRLF+CRLF;
    Stream3 := TFileStream.Create(Attachments[i], fmOpenRead);
    try
      SetLength(s, Stream3.Size);
      Stream3.ReadBuffer(PRVAnsiChar(s)^, Stream3.Size);
    except
      s := '(Cannot load file)';
    end;
    Stream3.Free;
    s := MimeEncodeString(s);
    Result := Result + s2+s+CRLF;
  end;

  if Attachments.Count>0 then
    Result := Result+CRLF+'--'+BOUNDARY_MAIN_AND_ATTACHMENTS+'--'+CRLF;
  HTMLImageCount := HTMLImages.Count;
  HTMLImages.Free;
end;
{------------------------------------------------------------------------------}
procedure TfrmMain.AddMainContentType(SL: TStrings);
begin
  if Attachments.Count>0 then begin
    SL.Add('Content-Type: multipart/mixed;');
    SL.Add(#9'boundary="'+BOUNDARY_MAIN_AND_ATTACHMENTS+'"');
    end
  else if HTMLImageCount>0 then begin
    SL.Add('Content-Type: multipart/related;');
    SL.Add(#9'type="multipart/alternative";');
    SL.Add(#9'boundary="'+BOUNDARY_MESSAGE_AND_PICTURES+'"');
    end
  else begin
    SL.Add('Content-Type: multipart/alternative;');
    SL.Add(#9'boundary="'+BOUNDARY_TEXT_AND_HTML+'"');
  end;
end;
{------------------------------------------------------------------------------}
{ Sending using Indy components }
procedure TfrmMain.btnSendClick(Sender: TObject);
var i: Integer;
begin
  btnSend.Enabled := False;
  btnSend.Caption := 'Wait...';
  try
    Attachments.Clear;
    for i := 0 to txtFiles.Lines.Count-1 do
      if Trim(txtFiles.Lines[i])<>'' then
        Attachments.Add(Trim(txtFiles.Lines[i]));

    IdMessage1.Clear;

    IdMessage1.From.Address := txtFromEMail.Text;
    IdMessage1.From.Name := txtFromName.Text;

    with IdMessage1.Recipients.Add do begin
      Address := txtToEMail.Text;
      Name := txtToName.Text;
    end;

    IdMessage1.Body.Text := GetEMail;
    IdMessage1.Subject := txtSubject.Text;

    with IdMessage1.ReplyTo.Add do begin
      Address := txtFromEMail.Text;
      Name := txtFromName.Text;
    end;

    // This demo does not use MIME features of Indy.
    // TIdSmtp thinks that it sends a plain text message.
    // Instead of using MIME features of Indy, we add ExtraHeaders and
    // assign MIME-encoded document to Message.Body.Text

    IdMessage1.ExtraHeaders.Add('MIME-Version: 1.0');
    // It's important to call AddMainContentType after GetEMail,
    // when HTMLImageCount is assigned
    AddMainContentType(IdMessage1.ExtraHeaders);

    IdSMTP1.MailAgent := 'Demo HTML Mailer';
    IdSMTP1.AuthenticationType := atLogin;
    IdSMTP1.Host := txtHost.Text;
    IdSMTP1.Port := StrToIntDef(txtPort.Text, 25);
    IdSMTP1.UserId := txtUserID.Text;
    IdSMTP1.Password := txtPassword.Text;

    IdSMTP1.Connect;
    IdSMTP1.Send(IdMessage1);
    IdSMTP1.Disconnect;

    ShowMessage('Success!');
  finally
    btnSend.Enabled := True;
    btnSend.Caption := 'Send';
  end;
end;

end.