
{*******************************************************}
{                                                       }
{       RichView                                        }
{       Demo: how to create MIME document.              }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************}


{
  This demo shows how to create MIME (Multipurpose Internet Mail Extensions)
  document.
  These documents contain both HTML and images in one file.
  File extension:
  *.eml - e-mail file, can be opened, for example, by Outlook Express.
  *.mht - web archive, can be opened, for example, by Internet Explorer.
  These files are essentially the same.

  This demo uses The Delphi Inspiration's DIMime unit for base64 encoding
  (freeware).
  Note: usually, base64 is used only for images and attachments. The main
  document is encoded as quoted-printable.
  For the simplicity reason, this demo encodes ALL parts as base64
  (thus making HTML part unreadable by humans).

  DIMime (c) Copyright Ralf Junker
  http://www.yunqa.de/delphi/doku.php/products/mime/
  Installation is included in this demo.

  Additional information about MIME:
  RFC 2045
  http://www.ietf.org/rfc/rfc2045.txt
  RFC 2046
  http://www.ietf.org/rfc/rfc2046.txt
  RFC 2047
  http://www.ietf.org/rfc/rfc2047.txt
  RFC 2048
  http://www.ietf.org/rfc/rfc2048.txt
  RFC 2049
  http://www.ietf.org/rfc/rfc2049.txt
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, RVStyle, RVScroll, RichView, RVEdit, CRVData,
  DIMime,
  Jpeg, ExtCtrls, RVTypes;

type

  { This class will be used to store information about one picture while
    saving to HTML }
  THTMLImageItem = class (TCollectionItem)
    public
      Stream: TMemoryStream;
      Name, ContentType: String;
      constructor Create(Collection: TCollection); override;
      destructor Destroy; override;
  end;
  { Collection of THTMLImageItem }
  THTMLImagesCollection = class (TCollection)
  private
    function GetItem(Index: Integer): THTMLImageItem;
    procedure SetItem(Index: Integer; const Value: THTMLImageItem);
    public
      constructor Create;
      property Items[Index: Integer]: THTMLImageItem read GetItem write SetItem; default;
  end;

  TForm1 = class(TForm)
    RichViewEdit1: TRichViewEdit;
    RVStyle1: TRVStyle;
    Button1: TButton;
    SaveDialog1: TSaveDialog;
    Image1: TImage;
    procedure FormCreate(Sender: TObject);
    procedure RichViewEdit1SaveImage2(Sender: TCustomRichView;
      Graphic: TGraphic; SaveFormat: TRVSaveFormat; const Path,
      ImagePrefix: String; var ImageSaveNo: Integer; var Location: String;
      var DoDefault: Boolean);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
    HTMLImages: THTMLImagesCollection;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{ Actually, this demo converts all pictures to Jpegs, so only TJpegImage can
  occur here. But you can use RV_RegisterHTMLGraphicFormat to allow other
  graphic formats in HTML }
function GetImageContentType(gr: TGraphic): String;
begin
  if gr is TJpegImage then
    Result := 'image/jpeg'
  else if gr is TBitmap then
    Result := 'image/bmp'
  else if gr is TIcon then
    Result := 'image/x-icon'
  else if gr is TMetafile then
    Result := 'application/octet-stream'
  {
  else if gr is TPNGObject then
    Result := 'image/png'
  else if gr is TGifImage then
    Result := 'image/gif'
  }
  else
    raise Exception.Create('Unknown graphic format');
end;

{=============================== THTMLImageItem ===============================}
constructor THTMLImageItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  Stream := TMemoryStream.Create;
end;
{------------------------------------------------------------------------------}
destructor THTMLImageItem.Destroy;
begin
  Stream.Free;
  inherited;
end;
{============================ THTMLImagesCollection ===========================}
constructor THTMLImagesCollection.Create;
begin
  inherited Create(THTMLImageItem);
end;
{------------------------------------------------------------------------------}
function THTMLImagesCollection.GetItem(Index: Integer): THTMLImageItem;
begin
  Result := THTMLImageItem(inherited GetItem(Index));
end;
{------------------------------------------------------------------------------}
procedure THTMLImagesCollection.SetItem(Index: Integer;
  const Value: THTMLImageItem);
begin
  inherited SetItem(Index, Value);
end;
{================================= TForm1 =====================================}
procedure TForm1.FormCreate(Sender: TObject);
var bmp: TBitmap;
begin
  RichViewEdit1.Clear;
  RichViewEdit1.AddBreakEx(4, rvbsLine, clBlue);
  RichViewEdit1.AddNL('MIME File Test', 1, 1);
  bmp := TBitmap.Create;
  bmp.Assign(Image1.Picture.Graphic);
  RichViewEdit1.AddPictureEx('', bmp, 1, rvvaBaseline);
  RichViewEdit1.AddNL('This example shows how to save HTML and images in one file.', 0, 0);
  RichViewEdit1.AddBreakEx(4, rvbsLine, clBlue);
  RichViewEdit1.Format;
end;
{------------------------------------------------------------------------------}
{ RichViewEdit1.OnSaveImage2: storing image in the HTMLImages collection instead
  of files. }
procedure TForm1.RichViewEdit1SaveImage2(Sender: TCustomRichView;
  Graphic: TGraphic; SaveFormat: TRVSaveFormat; const Path,
  ImagePrefix: String; var ImageSaveNo: Integer; var Location: String;
  var DoDefault: Boolean);
var gr: TGraphic; 
    bmp: TBitmap; 
begin 
  if SaveFormat<>rvsfHTML then 
    exit; 
  if not (Graphic is TJPEGImage) and not RV_IsHTMLGraphicFormat(Graphic) then begin 
    bmp := TBitmap.Create; 
    try 
      bmp.Assign(Graphic); 
    except 
      bmp.Width := Graphic.Width; 
      bmp.Height := Graphic.Height;
      bmp.Canvas.Draw(0,0, Graphic);
    end;
    gr := TJPEGImage.Create;
    gr.Assign(bmp);
    bmp.Free;
    end
  else
    gr := Graphic;

  Location := Format('image%d.%s', [ImageSaveNo, GraphicExtension(TGraphicClass(gr.ClassType))]);
  inc(ImageSaveNo);
  with HTMLImages.Add as THTMLImageItem do 
  begin 
    gr.SaveToStream(Stream); 
    Name := Location; 
    ContentType := GetImageContentType(gr); 
  end; 
  Location := 'cid:'+Location; 
  DoDefault := False; 
  if gr<>Graphic then 
    gr.Free; 
end;
{------------------------------------------------------------------------------}
{
  This version of document creation is simplified.
  It does not allow saving attached files and plain text representation of HTML
  document.
  See the full version in ..\Indy demo.
}
procedure TForm1.Button1Click(Sender: TObject);
var Stream: TMemoryStream;
    F: TextFile;
    s: TRVRawByteString;
    boundary: TRVRawByteString;
    i: Integer;
begin
  if not SaveDialog1.Execute then
    exit;

  HTMLImages := THTMLImagesCollection.Create;

  // saving HTML
  Stream := TMemoryStream.Create;
  RichViewEdit1.SaveHTMLToStreamEx(Stream, '', 'Web Archive Demo', '', '', '', '',
    [rvsoUseCheckpointsNames, rvsoUTF8]);
  Stream.Position := 0;
  SetLength(s, Stream.Size);
  Stream.ReadBuffer(PRVAnsiChar(s)^, Length(s));
  s := MimeEncodeString(s);
  Stream.Free;
  // now s contains HTML file without images, base64-encoded. saving it in MIME
  boundary := '----=_BOUNDARY_LINE_';
  AssignFile(F, SaveDialog1.FileName);
  Rewrite(F);
  Writeln(F, 'From: <Saved by TRICHVIEW.COM Demo>');
  Writeln(F, 'Subject: MIME Test');
  Writeln(F, 'MIME-Version: 1.0');
  Writeln(F, 'Content-Type: multipart/related;');
  Writeln(F, #9'boundary="',boundary,'"');
  Writeln(F, 'X-Unsent: 1');
  Writeln(F);
  Writeln(F, 'This is a multi-part message in MIME format.');
  Writeln(F);
  Writeln(F, '--', boundary);
  Writeln(F, 'Content-Type: text/html;');
	Writeln(F, #9'charset="utf-8"');
  Writeln(F, 'Content-Transfer-Encoding: base64');
  Writeln(F);
  Writeln(F, s);
  // saving images
  for i := 0 to HTMLImages.Count-1 do begin
    Writeln(F);
    Writeln(F, '--', boundary);
    SetLength(s, HTMLImages[i].Stream.Size);
    HTMLImages[i].Stream.Position := 0;
    HTMLImages[i].Stream.ReadBuffer(PRVAnsiChar(s)^, Length(s));
    s := MimeEncodeString(s);
    Writeln(F, 'Content-Type: ', HTMLImages[i].ContentType);
    Writeln(F, #9'Name="', HTMLImages[i].Name, '"');
    Writeln(F, 'Content-Transfer-Encoding: base64');
    Writeln(F, 'Content-ID: <', HTMLImages[i].Name, '>');
    Writeln(F);
    Writeln(F, s);
  end;
  Writeln(F);
  Writeln(F, '--', boundary, '--');
  CloseFile(F);
  HTMLImages.Free;
end;

end.